# frozen_string_literal: true

require_relative 'mixin/argument_range_helper'
require_relative 'mixin/in_delta_mixin'
require_relative 'mixin/instance_of_assertion_handleable'
require_relative 'mixin/minitest_cop_rule'
require_relative 'mixin/minitest_exploration_helpers'
require_relative 'mixin/nil_assertion_handleable'
require_relative 'mixin/predicate_assertion_handleable'
require_relative 'minitest/assert_empty'
require_relative 'minitest/assert_empty_literal'
require_relative 'minitest/assert_equal'
require_relative 'minitest/assert_in_delta'
require_relative 'minitest/assert_operator'
require_relative 'minitest/assert_predicate'
require_relative 'minitest/assert_raises_compound_body'
require_relative 'minitest/assert_raises_with_regexp_argument'
require_relative 'minitest/assert_with_expected_argument'
require_relative 'minitest/assertion_in_lifecycle_hook'
require_relative 'minitest/assert_kind_of'
require_relative 'minitest/assert_nil'
require_relative 'minitest/assert_includes'
require_relative 'minitest/assert_instance_of'
require_relative 'minitest/assert_match'
require_relative 'minitest/assert_output'
require_relative 'minitest/assert_path_exists'
require_relative 'minitest/assert_respond_to'
require_relative 'minitest/assert_same'
require_relative 'minitest/assert_silent'
require_relative 'minitest/assert_truthy'
require_relative 'minitest/duplicate_test_run'
require_relative 'minitest/empty_line_before_assertion_methods'
require_relative 'minitest/focus'
require_relative 'minitest/non_executable_test_method'
require_relative 'minitest/redundant_message_argument'
require_relative 'minitest/return_in_test_method'
require_relative 'minitest/test_file_name'
require_relative 'minitest/global_expectations'
require_relative 'minitest/lifecycle_hooks_order'
require_relative 'minitest/literal_as_actual_argument'
require_relative 'minitest/multiple_assertions'
require_relative 'minitest/no_assertions'
require_relative 'minitest/no_test_cases'
require_relative 'minitest/non_public_test_method'
require_relative 'minitest/refute_empty'
require_relative 'minitest/refute_false'
require_relative 'minitest/refute_equal'
require_relative 'minitest/refute_in_delta'
require_relative 'minitest/refute_includes'
require_relative 'minitest/refute_instance_of'
require_relative 'minitest/refute_kind_of'
require_relative 'minitest/refute_match'
require_relative 'minitest/refute_nil'
require_relative 'minitest/refute_operator'
require_relative 'minitest/refute_path_exists'
require_relative 'minitest/refute_predicate'
require_relative 'minitest/refute_respond_to'
require_relative 'minitest/refute_same'
require_relative 'minitest/skip_ensure'
require_relative 'minitest/skip_without_reason'
require_relative 'minitest/test_method_name'
require_relative 'minitest/unreachable_assertion'
require_relative 'minitest/unspecified_exception'
require_relative 'minitest/useless_assertion'
